/*
 * Copyright 2011 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.unitesk.requality.eclipse;

import java.util.HashMap;
import java.util.Map;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IExtensionRegistry;
import org.eclipse.core.runtime.Platform;
import org.eclipse.ui.IEditorReference;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;

import com.unitesk.requality.core.TreeNode;
import com.unitesk.requality.eclipse.editors.browser.ReqMarker;
import com.unitesk.requality.eclipse.tools.TreesTracker;

public class RequalityPlugin {
    private static Map<String, NodeTypeDescriptor> nodeTypes;

    public static TreesTracker getTreesTracker() {
        return TreesTracker.getTracker();
    }

    public static IEditorReference findMarkEditorForDocument(String docQId,
            IProject project) throws PartInitException {
        IEditorReference[] eRefs = PlatformUI.getWorkbench()
                .getActiveWorkbenchWindow().getActivePage()
                .getEditorReferences();
        for (IEditorReference eRef : eRefs) {
            if (!(eRef.getEditor(false) instanceof ReqMarker)) {
                continue;
            }
            ReqMarker marker = (ReqMarker) eRef.getEditor(true);
            if (!project.equals(TreesTracker.getResourceByNode(
                    marker.getDocument()).getProject())) {
                continue;
            }
            if (marker.getDocument().getQualifiedId().equals(docQId)) {
                return eRef;
            }
        }
        return null;
    }

    public static Map<String, NodeTypeDescriptor> getAdditionalTypes() {
        if (nodeTypes == null) {
            nodeTypes = new HashMap<String, NodeTypeDescriptor>();
            IExtensionRegistry reg = Platform.getExtensionRegistry();
            IConfigurationElement[] configurationElements = reg
                    .getConfigurationElementsFor(Activator.EXTENSION_NAME_NODE_TYPES);
            for (IConfigurationElement conf : configurationElements) {
                try {
                    TreeNode nt = (TreeNode) conf
                            .createExecutableExtension("typeclass");
                    nodeTypes.put(conf.getAttribute("name"),
                            new NodeTypeDescriptor(conf.getAttribute("name"),
                                    nt.getClass(), conf.getAttribute("icon"),
                                    conf.getAttribute("editor-id"), conf
                                            .getContributor().getName()));
                } catch (Exception e) {
                    e.printStackTrace();
                }
            }
        }

        return nodeTypes;
    }
}
