/*
 * Copyright 2009 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.unitesk.requality.eclipse.editors.browser;

import java.io.BufferedWriter;
import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.io.UnsupportedEncodingException;
import java.io.Writer;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.List;
import java.util.UUID;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IFolder;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IMenuListener;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.swt.SWT;
import org.eclipse.swt.browser.BrowserFunction;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.events.KeyListener;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.MouseListener;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.DirectoryDialog;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IEditorSite;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.part.EditorPart;
import org.eclipse.ui.part.FileEditorInput;
import org.eclipse.ui.part.IContributedContentsView;
import org.eclipse.ui.views.contentoutline.IContentOutlinePage;
import org.eclipse.ui.views.properties.IPropertySheetPage;
import org.eclipse.ui.views.properties.tabbed.ITabbedPropertySheetPageContributor;
import org.eclipse.ui.views.properties.tabbed.TabbedPropertySheetPage;

import com.unitesk.requality.core.ITreeChangeListener;
import com.unitesk.requality.core.TreeDB;
import com.unitesk.requality.core.TreeNode;
import com.unitesk.requality.eclipse.Activator;
import com.unitesk.requality.eclipse.RequalityPlugin;
import com.unitesk.requality.eclipse.core.ResourceStorage;
import com.unitesk.requality.eclipse.editors.NodeEditorInput;
import com.unitesk.requality.eclipse.tools.DefaultTreeChangeListener;
import com.unitesk.requality.eclipse.tools.ITreeStateListener;
import com.unitesk.requality.eclipse.tools.TreesTracker;
import com.unitesk.requality.nodetypes.Document;
import com.unitesk.requality.nodetypes.Location;
import com.unitesk.requality.nodetypes.Requirement;

public class ReqMarker extends EditorPart implements
        ITabbedPropertySheetPageContributor {
    private static final String DEFAULT_BASE_ENCODING = "UTF-8";
    public static final String ID = "com.unitesk.requality.editors.browser.ReqMarker";

    protected ReqDocBrowser browser;

    private Document doc;
    private String docQId;
    private TreeDB db;
    private Location initialLocation;
    private IProject project;

    public void openDocument(URL url) {
        String path = url.toString();
        browser.setUrl(path);
    }

    public ReqDocBrowser getBrowser() {
        return browser;
    }

    private ISelectionChangedListener locChangeListener = new ISelectionChangedListener() {

        @Override
        public void selectionChanged(SelectionChangedEvent event) {
            if (browser != null && !browser.isDisposed()) {
                IStructuredSelection s = (IStructuredSelection) event
                        .getSelection();
                Location loc = (Location) s.getFirstElement();
                browser.setTargetLocation(loc);
            }
        }
    };
    private ITreeChangeListener treeChangeListener = new DefaultTreeChangeListener() {
        public void deleted(TreeNode node) {
            if (docQId.equals(node.getQualifiedId())) {
                getSite().getShell().getDisplay().syncExec(new Runnable() {
                    @Override
                    public void run() {
                        getEditorSite().getPage().closeEditor(ReqMarker.this,
                                false);
                    }
                });
            }
            if (db.getParentQualifiedId(node.getQualifiedId()).equals(docQId)) {
                if ((TreesTracker.getResourceByNode(doc)).isAccessible()) {
                    if (reqMarkerOutlinePage != null
                            && !reqMarkerOutlinePage.getControl().isDisposed()) {
                        reqMarkerOutlinePage.refresh();
                    }
                }
            }
        }

        @Override
        public void changed(TreeNode node) {
            getSite().getShell().getDisplay().asyncExec(new Runnable() {
                @Override
                public void run() {
                    if (ReqMarker.this.getBrowser().isDisposed())
                        return;
                    if (TreesTracker.getTracker().getProject(db) == null) {
                        getEditorSite().getPage().closeEditor(ReqMarker.this,
                                false);
                        return;
                    }
                    browser.updateLocations();
                    if (reqMarkerOutlinePage != null
                            && !reqMarkerOutlinePage.getControl().isDisposed()) {
                        reqMarkerOutlinePage.refresh();
                    }
                }
            });
        }
    };

    private ITreeStateListener dbStateListener = new ITreeStateListener() {

        @Override
        public void open(TreeDB db) {
        }

        @Override
        public void close(TreeDB db) {
            getSite().getShell().getDisplay().asyncExec(new Runnable() {
                @Override
                public void run() {
                    getEditorSite().getPage()
                            .closeEditor(ReqMarker.this, false);
                }
            });
        }

        @Override
        public void delete(IProject proj) {
            if (project.equals(proj)) {
                getSite().getShell().getDisplay().asyncExec(new Runnable() {
                    @Override
                    public void run() {
                        getEditorSite().getPage().closeEditor(ReqMarker.this,
                                false);
                    }
                });
            }
        }
    };

    private ReqMarkerOutlinePage reqMarkerOutlinePage;

    final class BrowserKeyListener implements KeyListener {
        @Override
        public void keyReleased(KeyEvent e) {
            if ((e.stateMask & SWT.CTRL) != 0) {
                if (e.keyCode == 'r')
                    new AddSelectionToRequirement(ReqMarker.this).run();
                if (e.keyCode == 'd')
                    new RemoveLocationFromRequirement(ReqMarker.this).run();
            }
        }

        @Override
        public void keyPressed(KeyEvent e) {
        }
    }

    // перехватываем более низкоуровневые события, чем
    // IMenuListener.menuAboutToShow.
    // При событии mouseDown запрещаем выполнение
    // IMenuListener.menuAboutToShow,
    // (чтобы событие Right Mouse Down было передано браузеру),
    // но затем при событии mouseUp вызываем IMenuListener.menuAboutToShow
    // искусственно
    // с помощью Menu.setVisible(true)
    // Сделано это, чтобы событие Mouse Click (Right) передавалось в
    // JavaScript браузера.
    final class BrowserMouseListener implements MouseListener {
        @Override
        public void mouseDown(MouseEvent e) {
            if (e.button == 3)
                allow_popup_menu = false;
        }

        @Override
        public void mouseUp(MouseEvent e) {
            if (e.button == 1) {
                browser.activateSelectedLocation();
            } else if (e.button == 3) {
                allow_popup_menu = true;
                browser.getWebBrowser().getMenu().setVisible(true);
            }
        }

        @Override
        public void mouseDoubleClick(MouseEvent e) {
        }
    }

    public ReqMarker() {
    }

    protected void updateTargetRequirement() {
        List<Location> locationList = getTargetRequirement().getLocations();
        if (locationList.size() == 0)
            return;
        Location location = locationList.get(0);

        setTargetLocation(location);
    }

    public void setTargetLocation(Location loc) {
        if (browser == null || loc == null) {
            return;
        }
        browser.setTargetLocation(loc);
        if (reqMarkerOutlinePage != null
                && !reqMarkerOutlinePage.getControl().isDisposed()) {
            reqMarkerOutlinePage.setSelection(new StructuredSelection(loc));
        }
    }

    public void saveDocument() throws UnsupportedEncodingException {
        String htmlContent = browser.serializeDocument();
        doc.resetCache();
        if (htmlContent != null && htmlContent.length() > 0) {
            try {
                ResourceStorage rs = (ResourceStorage) db.getStorage();
                IFolder folder = rs.getResourceFolder(doc.getUUId());
                IFile f = folder.getFile(doc.getResourceName());
                ByteArrayInputStream in = new ByteArrayInputStream(
                        htmlContent.getBytes(DEFAULT_BASE_ENCODING));
                f.setContents(in, false, true, null);
            } catch (CoreException e) {
                e.printStackTrace();
            }
        }
    }

    @Override
    public void doSave(IProgressMonitor monitor) {
    }

    @Override
    public void doSaveAs() {
        throw new UnsupportedOperationException("Unexpected method call");
    }

    @Override
    public void init(IEditorSite site, IEditorInput input)
            throws PartInitException {
        try {
            setSite(site);
            if (input instanceof NodeEditorInput) {
                NodeEditorInput docInput = (NodeEditorInput) input;
                if (!docInput.getStorage().getNode().getType()
                        .equals("Document")
                        && !docInput.getStorage().getNode().getType()
                                .equals("Requirement")) {
                    throw new PartInitException("Wrong input type: "
                            + docInput.getStorage().getNode().getType());
                }
                if (docInput.getStorage().getNode() instanceof Requirement) {
                    Requirement req = (Requirement) docInput.getStorage()
                            .getNode();
                    doc = (Document) req.getLocations().get(0).getParent();
                    initialLocation = (Location) req.getLocations().get(0);
                } else {
                    doc = (Document) docInput.getStorage().getNode();
                    initialLocation = (Location) docInput.getData();
                }
                db = docInput.getStorage().getNode().getTreeDB();
                project = RequalityPlugin.getTreesTracker().getProject(db);
                docQId = doc.getQualifiedId();
            } else if (input instanceof FileEditorInput) {
                MessageDialog
                        .openError(getSite().getShell(),
                                "Unsupported editor open method",
                                "not yet implemented");
            }
        } catch (CoreException e) {
            e.printStackTrace();
        }
        setInput(input);
        setPartName(doc.getId());
        db.addListener(treeChangeListener);
        TreesTracker.getTracker().addListener(dbStateListener);
    }

    @Override
    public boolean isDirty() {
        return false;
    }

    @Override
    public boolean isSaveAsAllowed() {
        return false;
    }

    @Override
    public void createPartControl(Composite parent) {
        parent.setLayout(new GridLayout(1, false));

        Composite browserPanel = new Composite(parent, SWT.NONE);
        browserPanel.setLayoutData(new GridData(GridData.FILL_BOTH));
        browserPanel.setLayout(new FillLayout());

        browser = new ReqDocBrowser(browserPanel, SWT.NONE);

        browser.getWebBrowser().setMenu(createMenu());

        new BrowserFunction(browser.getWebBrowser(), "setInitialLocation") {
            @Override
            public Object function(Object[] arguments) {
                setTargetLocation(initialLocation);
                return null;
            }
        };

        browser.getWebBrowser().addKeyListener(new BrowserKeyListener());

        browser.getWebBrowser().addMouseListener(new BrowserMouseListener());
        // Get path to file

        // TODO: check for resource existing
        String openPath = doc.getHTMLPath();
        try {
            URL url = new File(openPath).toURI().toURL();
            openDocument(url);
        } catch (MalformedURLException e1) {
            e1.printStackTrace();
        }
    }

    private boolean allow_popup_menu;

    private Menu createMenu() {
        MenuManager menuMgr = new MenuManager("#PopupMenu");
        menuMgr.setRemoveAllWhenShown(true);

        menuMgr.addMenuListener(new IMenuListener() {
            // menuAboutToShow вызывается 2 раза:
            // при mouseDown запрещаем создание меню, при mouseUp разрешаем
            public void menuAboutToShow(IMenuManager manager) {
                if (!allow_popup_menu)
                    return;
                manager.add(new AddSelectionToRequirement(ReqMarker.this));

                Requirement req = getTargetRequirement();
                if (req != null) {
                    if (req.getQualifiedId().equals(
                            Requirement.getTypeRootQId())) {
                        manager.add(new AddSelectionToRequirement(
                                ReqMarker.this, req, AddSelectionToRequirement
                                        .getNextId(req)));
                    } else {
                        manager.add(new AddSelectionToRequirement(
                                ReqMarker.this, req));

                        manager.add(new AddSelectionToRequirement(
                                ReqMarker.this, req, AddSelectionToRequirement
                                        .getNextId(req)));
                    }
                } else {
                    req = (Requirement) db.getNode(Requirement.getTypeRootQId());
                    manager.add(new AddSelectionToRequirement(ReqMarker.this,
                            req, AddSelectionToRequirement.getNextId(req)));
                }

                String newName = browser.getSelectionText().trim();
                if (newName.length() > 0 && newName.length() < 128) {
                    manager.add(new AddSelectionToRequirement(ReqMarker.this,
                            req, AddSelectionToRequirement.getNextId(req),
                            newName));
                }

                manager.add(new RemoveLocationFromRequirement(ReqMarker.this));

                manager.add(new Action("Show Version") {
                    @Override
                    public void run() {
                        browser.showBrowserInfo();
                    }
                });

                manager.add(new Action("Export Page...") {
                    @Override
                    public void run() {
                        String htmlContent = browser.serializeDocumentRaw();
                        htmlContent = htmlContent.replace(Activator
                                .getDefault().getJSToolsPath(), "");
                        DirectoryDialog dialog = new DirectoryDialog(getSite()
                                .getShell());
                        String path = dialog.open();
                        if (path != null) {
                            try {
                                copyPageFile(path + File.separator);
                                Writer out = new BufferedWriter(
                                        new OutputStreamWriter(
                                                new FileOutputStream(path
                                                        + File.separator
                                                        + doc.getId()), "UTF8"));
                                out.write(htmlContent);
                                out.close();
                            } catch (IOException e) {
                                e.printStackTrace();
                            }
                        }
                    }

                    private void copyPageFile(String path) throws IOException {
                        String jsPath = Activator.getDefault().getJSToolsPath();
                        new File(path + File.separator + "jquery").mkdirs();
                        copyFile(new File(jsPath + File.separator + "jquery"
                                + File.separator + "jquery.js"), new File(path
                                + File.separator + "jquery" + File.separator
                                + "jquery.js"));
                        copyFile(new File(jsPath + "edit.js"), new File(path
                                + "edit.js"));
                        copyFile(new File(jsPath + "edit.css"), new File(path
                                + "edit.css"));
                    }
                });

            }
        });

        Menu menu = menuMgr.createContextMenu(browser);
        return menu;
    }

    public static void copyFile(File in, File out) throws IOException {
        FileInputStream fis = new FileInputStream(in);
        FileOutputStream fos = new FileOutputStream(out);
        try {
            byte[] buf = new byte[1024];
            int i = 0;
            while ((i = fis.read(buf)) != -1) {
                fos.write(buf, 0, i);
            }
        } catch (IOException e) {
            throw e;
        } finally {
            if (fis != null)
                fis.close();
            if (fos != null)
                fos.close();
        }
    }

    @Override
    public void setFocus() {
        browser.setFocus();
    }

    public TreeDB getTreeDB() {
        return db;
    }

    public Document getDocument() {
        return doc;
    }

    public boolean addSelectionTo(Requirement req, boolean hideLocation) {
        String locId = UUID.randomUUID().toString();
        String reqQId = req.getQualifiedId();
        String desc = browser.createLocation(locId, reqQId);
        req.addLocation(doc.getQualifiedId() + "/" + locId);
        req.saveAttributes();
        try {
            saveDocument();
            TreesTracker.getResourceByNode(doc).refreshLocal(
                    IResource.DEPTH_INFINITE, null);
        } catch (Exception e) {
            e.printStackTrace();
        }
        Location loc = (Location) doc.createChild(locId, "Location");
        loc.setDescription(desc);
        if (hideLocation) {
            loc.isHidden(true);
        }
        loc.saveAttributes();
        return true;
    }

    public void removeLocation(String locationId) {
        removeLocation(locationId, false);
    }

    public void removeLocation(String locationId, boolean forse) {
        String locQId = doc.getQualifiedId() + "/" + locationId;
        Requirement[] reqs = getTreeDB().getRequirementsFor(locQId);
        for (Requirement req : reqs) {
            req.removeLocation(locQId);
            if (!forse && req.getLocations().size() == 0
                    && req.sizeChildren() == 0) {
                if (MessageDialog.openQuestion(getSite().getShell(),
                        "Delete Requirement", "Requirement " + req.getName()
                                + " has no locations, delete?")) {
                    req.getParent().removeChild(req);
                    break;
                }
            }
            req.saveAttributes();
        }
        browser.deleteLocation(locationId);
        try {
            saveDocument();
            TreesTracker.getResourceByNode(doc).refreshLocal(
                    IResource.DEPTH_INFINITE, null);
        } catch (Exception e) {
            e.printStackTrace();
        }
        Location loc = (Location) getTreeDB().getNode(locQId);
        loc.delete();
    }

    @Override
    public void dispose() {
        db.removeListener(treeChangeListener);
        TreesTracker.getTracker().removeListener(dbStateListener);
        super.dispose();
    }

    public Requirement getTargetRequirement() {
        Requirement req = (Requirement) Activator.getDefault().getActiveNode(
                "Requirement");
        if (req != null && req.getTreeDB() == doc.getTreeDB()) {
            return req;
        }
        return null;
    }

    public void close(boolean save) {
        getSite().getPage().closeEditor(this, save);
    }

    @Override
    public Object getAdapter(@SuppressWarnings("rawtypes") Class required) {
        if (IContentOutlinePage.class.equals(required)) {
            if (reqMarkerOutlinePage == null) {
                reqMarkerOutlinePage = new ReqMarkerOutlinePage();
                reqMarkerOutlinePage.setInput(doc);
                reqMarkerOutlinePage
                        .addSelectionChangedListener(locChangeListener);
            }
            return reqMarkerOutlinePage;
        }

        if (required == IPropertySheetPage.class) {
            return new TabbedPropertySheetPage(this);
        }
        if (required == IContributedContentsView.class) {
            return new IContributedContentsView() {
                public IWorkbenchPart getContributingPart() {
                    return PlatformUI
                            .getWorkbench()
                            .getActiveWorkbenchWindow()
                            .getActivePage()
                            .findView(
                                    "com.unitesk.requality.views.RequalityExplorer");
                }
            };

        }

        return super.getAdapter(required);
    }

    @Override
    public String getContributorId() {
        return "com.unitesk.requality.views.RequalityPropertyContributor";
    }
}
