/*
 * Copyright 2011 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.unitesk.requality.eclipse.handlers.report;

import java.io.File;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

import org.eclipse.core.commands.AbstractHandler;
import org.eclipse.core.commands.ExecutionEvent;
import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.commands.IHandler;
import org.eclipse.core.resources.IContainer;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IFolder;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.Path;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.InputDialog;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.PlatformUI;

import com.unitesk.requality.core.Attribute;
import com.unitesk.requality.core.AttributeType;
import com.unitesk.requality.core.TreeDB;
import com.unitesk.requality.core.TreeNode;
import com.unitesk.requality.core.instancer.TreeInstancer;
import com.unitesk.requality.eclipse.Activator;
import com.unitesk.requality.eclipse.NodeTypeDescriptor;
import com.unitesk.requality.eclipse.RequalityPlugin;
import com.unitesk.requality.eclipse.tools.TreesTracker;
import com.unitesk.requality.eclipse.ui.cnf.RequalityCNF;
import com.unitesk.requality.nodetypes.ReportData;
import com.unitesk.requality.nodetypes.ReportFolder;
import com.unitesk.requality.nodetypes.Requirement;
import com.unitesk.requality.report.ReportGenerator;
import com.unitesk.requality.report.Template;

public class GenerateReportHandler extends AbstractHandler implements IHandler {
    public static IContainer getReportFolder() {
        TreeNode node = RequalityCNF.getSelectedNode();
        IFile nodeFile = (IFile) TreesTracker.getResourceByNode(node);
        IContainer res = nodeFile.getParent().getFolder(
                Path.fromOSString(node.getId() + "_resources"));
        return res;
    }

    @Override
    public Object execute(ExecutionEvent event) throws ExecutionException {
        TreeNode node = RequalityCNF.getSelectedNode();
        ReportData report = null;
        if (node instanceof ReportData) {
            report = (ReportData) node;
        } else if (node instanceof Requirement) {
            Requirement req = (Requirement) node;
            Shell shell = PlatformUI.getWorkbench().getActiveWorkbenchWindow()
                    .getShell();
            // TODO: validator
            InputDialog dialog = new InputDialog(shell, "Input Report Name",
                    "Input Report Name", req.getId() + "-report", null);
            if (dialog.open() == Dialog.OK) {
                ReportFolder folder = (ReportFolder) req.getTreeDB().getNode(
                        "/Reports");
                report = (ReportData) folder.createChild(dialog.getValue(),
                        "ReportData");

            }
        }
        if (report != null) {
        	generateReport(report);
            OpenReportHandler.openReport(report);
        }
        return null;
    }

    public static String generateReport(ReportData node) {
        TreeDB db = node.getTreeDB();
        Requirement req = (Requirement) db.getNode(node
                .getStartRequirementQId());
        TreeDB instTree = new TreeDB(new TreeInstancer(db, node.getData()));
        Map<String, NodeTypeDescriptor> nts = RequalityPlugin
                .getAdditionalTypes();
        for (String name : RequalityPlugin.getAdditionalTypes().keySet()) {
            instTree.addNodeType(name, nts.get(name).getClazz());
        }

        String href = null;
        try {
            Template tpl = getTemplate(node.getTemplateId());
            if (tpl == null) {
                tpl = Activator.getDefault().getReportTemplates().values()
                        .iterator().next();
            }
            String savePath = node.getSaveDir();
            ReportGenerator.generateReport(
                    (Requirement) instTree.getNode(req.getQualifiedId()), tpl,
                    savePath, new HashMap<String, Object>(node.getData()));
            // TODO: index file selection
            href = savePath + File.separator
                    + tpl.getTemplates().get(0).getOutputFileName();

            IProject project = TreesTracker.getTracker().getProject(db);
            IPath path = Path.fromOSString(savePath).makeRelativeTo(
                    project.getLocation());

            IFolder folder = project.getFolder(path);
            folder.refreshLocal(IResource.DEPTH_INFINITE, null);
        } catch (Exception e) {
            e.printStackTrace();
        }
        node.putAttribute(new Attribute(AttributeType.STRING, "date", new Date(
                System.currentTimeMillis()).toString()));
        node.saveAttributes();
        return href;
    }

    public static Template getTemplate(String tid) {
        return Activator.getDefault().getReportTemplates().get(tid);
    }
}
