/*
 * Copyright 2011 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.unitesk.requality.eclipse.handlers.report;

import java.io.File;

import org.eclipse.core.commands.AbstractHandler;
import org.eclipse.core.commands.ExecutionEvent;
import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.resources.IContainer;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.Path;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.browser.IWebBrowser;
import org.eclipse.ui.ide.IDE;

import com.unitesk.requality.core.TreeNode;
import com.unitesk.requality.eclipse.Activator;
import com.unitesk.requality.eclipse.editors.NodeEditorInput;
import com.unitesk.requality.eclipse.editors.browser.ReportViewer;
import com.unitesk.requality.eclipse.tools.TreesTracker;
import com.unitesk.requality.eclipse.ui.cnf.RequalityCNF;
import com.unitesk.requality.nodetypes.ReportData;
import com.unitesk.requality.report.Template;

public class OpenReportHandler extends AbstractHandler {
    public Object execute(ExecutionEvent event) throws ExecutionException {
        ReportData rdata = (ReportData) RequalityCNF.getSelectedNode();
        Template tpl = Activator.getDefault().getReportTemplates()
                .get(rdata.getTemplateId());
        if (tpl == null) {
            // get first
            tpl = Activator.getDefault().getReportTemplates().values()
                    .iterator().next();
        }
        IResource res = OpenReportHandler.getReportFolder()
                .getFile(
                        Path.fromOSString(tpl.getTemplates().get(0)
                                .getOutputFileName()));
        if (!res.exists()) {
            GenerateReportHandler.generateReport(rdata);
        }

        openReport(rdata);
        return null;
    }

    public static IWebBrowser openReport(ReportData rdata) {
        IWorkbenchPage page = PlatformUI.getWorkbench()
                .getActiveWorkbenchWindow().getActivePage();
        Template tpl = Activator.getDefault().getReportTemplates()
                .get(rdata.getTemplateId());
        if (tpl == null) {
            // get first
            tpl = Activator.getDefault().getReportTemplates().values()
                    .iterator().next();
        }
        String savePath = rdata.getSaveDir();
        String href = savePath + File.separator
                + tpl.getTemplates().get(0).getOutputFileName();
        if ((page != null) && (href != null) && (href.length() > 0)) {
            try {
                NodeEditorInput input = new NodeEditorInput(rdata);
                IDE.openEditor(PlatformUI.getWorkbench()
                        .getActiveWorkbenchWindow().getActivePage(), input,
                        ReportViewer.ID, false);
            } catch (PartInitException e) {
                e.printStackTrace();
            }
        }
        return null;
    }

    public static IContainer getReportFolder() {
        TreeNode node = RequalityCNF.getSelectedNode();
        IFile nodeFile = (IFile) TreesTracker.getResourceByNode(node);
        IContainer res = nodeFile.getParent().getFolder(
                Path.fromOSString(node.getId() + "_resources"));
        return res;
    }
}
