/*
 * Copyright 2011 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.unitesk.requality.eclipse.handlers.requirement;

import java.io.UnsupportedEncodingException;

import org.eclipse.core.commands.AbstractHandler;
import org.eclipse.core.commands.ExecutionEvent;
import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.ui.IEditorReference;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;

import com.unitesk.requality.core.TreeNode;
import com.unitesk.requality.eclipse.Activator;
import com.unitesk.requality.eclipse.RequalityPlugin;
import com.unitesk.requality.eclipse.editors.browser.ReqMarker;
import com.unitesk.requality.eclipse.tools.TreesTracker;
import com.unitesk.requality.eclipse.ui.cnf.RequalityCNF;
import com.unitesk.requality.nodetypes.Document;
import com.unitesk.requality.nodetypes.Location;
import com.unitesk.requality.nodetypes.Requirement;

public class RemoveRequirementHandler extends AbstractHandler {

    @Override
    public Object execute(ExecutionEvent event) throws ExecutionException {
        TreeNode[] nodes = new TreeNode[1];
        nodes[0] = RequalityCNF.getSelectedNode();
        TreeNode parent = nodes[0].getParent();
        if (nodes.length == 1) {
            if (!MessageDialog.openConfirm(PlatformUI.getWorkbench()
                    .getActiveWorkbenchWindow().getShell(),
                    "Remove Requirement", "Remove selected "
                            + nodes[0].getType().toLowerCase() + "?")) {
                return null;
            }
        } else {
            if (!MessageDialog.openConfirm(PlatformUI.getWorkbench()
                    .getActiveWorkbenchWindow().getShell(), "Remove Nodes",
                    "Remove Selected nodes")) {
                return null;
            }
        }
        for (TreeNode node : nodes) {
            if (!(node.getParent() instanceof Requirement))
                continue;
            if (node.getType().equals("Requirement")) {
                deleteLocations((Requirement) node);
            }
            node.delete();
        }
        if (parent != null)
            Activator.getDefault().setActiveNode(parent, parent.getType());
        return null;
    }

    // TODO: (low) check for crosslinks to locations
    private void deleteLocations(Requirement req) {
        for (Location loc : req.getLocations()) {
            if (loc == null) {
                // FIXME:
                continue;
            }
            try {
                Document doc = (Document) loc.getParent();
                if (doc == null)
                    return;
                IEditorReference eRef = RequalityPlugin.findMarkEditorForDocument(
                        doc.getQualifiedId(),
                        TreesTracker.getResourceByNode(doc).getProject());
                if (eRef != null) {
                    if (eRef.getEditor(true) != null
                            && (eRef.getEditor(true) instanceof ReqMarker)) {
                        ReqMarker marker = (ReqMarker) eRef.getEditor(true);
                        marker.getBrowser().deleteLocation(loc.getId());
                        marker.saveDocument();
                    }
                } else {
                    doc.removeLocation(loc.getId());
                }
                doc.removeChild(loc);
                TreesTracker.getResourceByNode(doc).refreshLocal(
                        IResource.DEPTH_INFINITE, null);
            } catch (PartInitException e) {
                e.printStackTrace();
            } catch (UnsupportedEncodingException e) {
                e.printStackTrace();
            } catch (CoreException e) {
                e.printStackTrace();
            }
        }
        for (int i = 0; i < req.sizeChildren(); i++) {
            TreeNode node = req.getChild(i);
            if (node instanceof Requirement) {
                Requirement r = (Requirement) node;
                deleteLocations(r);
            }
        }
    }

    @Override
    public boolean isEnabled() {
        TreeNode node = RequalityCNF.getSelectedNode();
        if (node == null)
            return false;
        if (node.getQualifiedId().equals(Requirement.getTypeRootQId()))
            return false;
        return super.isEnabled();
    }
}
