/*
 * Copyright 2011 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.unitesk.requality.eclipse.ui.cnf;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IEditorReference;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.navigator.ILinkHelper;

import com.unitesk.requality.core.TreeNode;
import com.unitesk.requality.eclipse.RequalityPlugin;
import com.unitesk.requality.eclipse.editors.NodeEditorInput;
import com.unitesk.requality.eclipse.tools.TreesTracker;
import com.unitesk.requality.nodetypes.Document;

public class CNFLinkHelper implements ILinkHelper {

    @Override
    public IStructuredSelection findSelection(IEditorInput anInput) {
        if (anInput instanceof NodeEditorInput) {
            NodeEditorInput input = (NodeEditorInput) anInput;
            try {
                TreeNode node = (TreeNode) input.getStorage().getNode();
                RequalityCNF view = (RequalityCNF) PlatformUI.getWorkbench()
                        .getActiveWorkbenchWindow().getActivePage()
                        .findView(RequalityCNF.ID);
                if (view != null) {
                    view.selectReveal(new StructuredSelection(node));
                }
            } catch (CoreException e) {
                e.printStackTrace();
            }
        }
        return null;
    }

    @Override
    public void activateEditor(IWorkbenchPage aPage,
            IStructuredSelection aSelection) {
        if (aSelection.getFirstElement() instanceof Document) {
            Document doc = (Document) aSelection.getFirstElement();
            IEditorReference eref = null;
            try {
                eref = RequalityPlugin.findMarkEditorForDocument(
                        doc.getQualifiedId(),
                        TreesTracker.getResourceByNode(doc).getProject());
            } catch (PartInitException e) {
                e.printStackTrace();
            }
            if (eref != null) {
                PlatformUI.getWorkbench().getActiveWorkbenchWindow()
                        .getActivePage().activate(eref.getEditor(true));
            }
        }
    }

}
