/*
 * Copyright 2011 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.unitesk.requality.eclipse.views.properties;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.FocusAdapter;
import org.eclipse.swt.events.FocusEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.views.properties.tabbed.TabbedPropertySheetPage;
import org.eclipse.ui.views.properties.tabbed.TabbedPropertySheetWidgetFactory;

import com.unitesk.requality.core.TreeNode;
import com.unitesk.requality.nodetypes.Requirement;

public class AdvancedPropertySection extends TreeNodePropertiesTab {
    private Text textPredicate;
    private BaseRequirementPanel panelBase;
    private Requirement req;

    public AdvancedPropertySection() {
    }

    @Override
    public void createControls(Composite parent,
            TabbedPropertySheetPage aTabbedPropertySheetPage) {
        super.createControls(parent, aTabbedPropertySheetPage);
        parent.setLayout(new GridLayout(1, false));
        parent.setLayoutData(new GridData(GridData.FILL_BOTH));
        TabbedPropertySheetWidgetFactory wf = aTabbedPropertySheetPage
                .getWidgetFactory();
        wf.createLabel(parent, "Predicate:");
        GridData predicateLData = new GridData(GridData.FILL_BOTH);
        predicateLData.widthHint = 0;
        predicateLData.heightHint = 0;
        textPredicate = wf.createText(parent, "true", SWT.BORDER | SWT.MULTI
                | SWT.H_SCROLL | SWT.V_SCROLL);
        textPredicate.setLayoutData(predicateLData);
        textPredicate.addFocusListener(new FocusAdapter() {
            public void focusLost(FocusEvent e) {
                req.setPredicate(textPredicate.getText());
                req.saveAttributes();
            }
        });
        panelBase = new BaseRequirementPanel(parent, this,
                aTabbedPropertySheetPage);
    }

    @Override
    protected void setTreeNode(TreeNode node) {
        req = toRequirement(node);

        if (req == null || textPredicate == null || textPredicate.isDisposed()) {
            return;
        }
        textPredicate.setText(req.getPredicate());
        panelBase.setRequirement(req);
    }
}
