/*
 * Copyright 2011 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.unitesk.requality.eclipse.views.properties;

import org.eclipse.jface.fieldassist.ControlDecoration;
import org.eclipse.jface.fieldassist.FieldDecoration;
import org.eclipse.jface.fieldassist.FieldDecorationRegistry;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.FocusAdapter;
import org.eclipse.swt.events.FocusEvent;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.layout.FormAttachment;
import org.eclipse.swt.layout.FormData;
import org.eclipse.swt.layout.FormLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.views.properties.tabbed.TabbedPropertySheetPage;
import org.eclipse.ui.views.properties.tabbed.TabbedPropertySheetWidgetFactory;

import com.unitesk.requality.core.TreeNode;
import com.unitesk.requality.nodetypes.Requirement;

public class MainPropertySection extends TreeNodePropertiesTab {

	private AttributesPanel attribPane;
	private Text textId;
	private Text textName;
	ControlDecoration reqIdDecorator;
    private Requirement req;

	public MainPropertySection() {
	}

	@Override
	public void createControls(Composite parent,
			TabbedPropertySheetPage aTabbedPropertySheetPage) {
		super.createControls(parent, aTabbedPropertySheetPage);
		TabbedPropertySheetWidgetFactory wf = getWidgetFactory();

		parent.setLayout(new FormLayout());
		parent.setLayoutData(new GridData(GridData.FILL_BOTH));
		FormAttachment topSpace1 = new FormAttachment(0, 8);
		FormAttachment topSpace2 = new FormAttachment(0, 32);
		FormAttachment leftSpace1 = new FormAttachment(0, 8);
		FormAttachment leftSpace2 = new FormAttachment(0, 50);
		FormAttachment leftSpace3 = new FormAttachment(0, 60);
		FormAttachment rightSpace1 = new FormAttachment(100, -8);

		FormData labelIdData = new FormData();
		labelIdData.top = topSpace1;
		labelIdData.right = leftSpace2;
		FormData labelNameData = new FormData();
		labelNameData.top = topSpace2;
		labelNameData.right = leftSpace2;
		FormData labelAttribData = new FormData();
		labelAttribData.top = new FormAttachment(0, 48);
		labelAttribData.left = leftSpace1;

		FormData textIdData = new FormData(0, SWT.DEFAULT);
		textIdData.top = topSpace1;
		textIdData.left = leftSpace3;
		textIdData.right = rightSpace1;

		FormData textNameData = new FormData(0, SWT.DEFAULT);
		textNameData.top = topSpace2;
		textNameData.left = leftSpace3;
		textNameData.right = rightSpace1;

		FormData panelData = new FormData();
		panelData.left = leftSpace1;
		panelData.right = rightSpace1;
		panelData.top = new FormAttachment(0, 64);
		panelData.bottom = new FormAttachment(100, -36);

		wf.createLabel(parent, "Id:").setLayoutData(labelIdData);
		textId = wf.createText(parent, "hello");
		textId.setLayoutData(textIdData);
		reqIdDecorator = createDecorator(textId);
		reqIdDecorator.hide();
		textId.addModifyListener(new ModifyListener() {

			@Override
			public void modifyText(ModifyEvent e) {
				if (isReqIdOk(textId.getText())) {
					reqIdDecorator.hide();
				} else {
					if(textId.getText().trim().equals("")) {
						reqIdDecorator.setDescriptionText("Requirement Id should not be empty");
					} else {
					reqIdDecorator.setDescriptionText("Requirement with same Id already exists");
					}
					reqIdDecorator.show();
				}				
			}
			
		});
		textId.addFocusListener(new FocusAdapter() {
			public void focusLost(FocusEvent e) {
				if (req.getId().equals(textId.getText())) {
					return;
				}
				if (!req.getQualifiedId().equals(Requirement.getTypeRootQId())) {
				    String newId = textId.getText().trim();
					if (isReqIdOk(newId)) {
						req.setId(newId.trim());
						if(!textName.getText().equals(req.getName())) {
						    textName.setText(req.getName());
						}
					} else {
						textId.setText(req.getId());
					}
				}
			}
		});
		wf.createLabel(parent, "Name:").setLayoutData(labelNameData);
		textName = wf.createText(parent, "node name");
		textName.setLayoutData(textNameData);
		textName.addFocusListener(new FocusAdapter() {
			public void focusLost(FocusEvent e) {
				if (req.getName().equals(textName.getText())) {
					return;
				}
                if (textName.getText().length() > 0) {
                    req.setName(textName.getText());
                } else {
                    textName.setText(req.getId());
                }
				req.saveAttributes();
			}
		});

		attribPane = new AttributesPanel(parent, wf);
		attribPane.getControl().setLayoutData(panelData);
	}

	private ControlDecoration createDecorator(Text text) {
		ControlDecoration controlDecoration = new ControlDecoration(text,
				SWT.LEFT | SWT.TOP);
		FieldDecoration fieldDecoration = FieldDecorationRegistry.getDefault()
				.getFieldDecoration(FieldDecorationRegistry.DEC_ERROR);
		controlDecoration.setImage(fieldDecoration.getImage());
		return controlDecoration;
	}

	protected boolean isReqIdOk(String text) {
		for (TreeNode req : this.req.getParent().getSortedChildren(
				"Requirement")) {
            if(text.trim().length()==0) {
                return false;
            }
			if (req.getId().equals(text.trim()) && !req.getUUId().equals(this.req.getUUId()))
				return false;
		}
		return true;
	}

	@Override
	public void refresh() {
		super.refresh();
		if (req != null) {
			textId.setText(req.getId());
			textName.setText(req.getName());
		}
	}

	@Override
	protected void setTreeNode(TreeNode node) {
	    req = toRequirement(node);
		if (req==null || textId == null || textId.isDisposed()) {
			return;
		}
		if (!req.getQualifiedId().equals(Requirement.getTypeRootQId()))
			textId.setEnabled(true);
		else
			textId.setEnabled(false);
		attribPane.setRequirement(req);
	}
}
