/*
 * Copyright 2009 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.unitesk.requality.eclipse.wizards;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IProjectDescription;
import org.eclipse.core.resources.IWorkspace;
import org.eclipse.core.resources.IWorkspaceRoot;
import org.eclipse.core.resources.IWorkspaceRunnable;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.wizard.Wizard;
import org.eclipse.ui.INewWizard;
import org.eclipse.ui.IPerspectiveDescriptor;
import org.eclipse.ui.IPerspectiveRegistry;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchWizard;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.WorkbenchException;

import com.unitesk.requality.core.TreeDB;
import com.unitesk.requality.core.TreeNode;
import com.unitesk.requality.eclipse.perspective.ReqEditorPerspectiveFactory;
import com.unitesk.requality.eclipse.tools.TreesTracker;

public class ReqWizard extends Wizard implements INewWizard {
    private static final String REQUALITY_PERSPECTIVE_PREFIX = "com.unitesk.requality.";
    private ReqWizardPage page;
    private ISelection selection;
    private IProject newProject;
    /**
     * Constructor for ReqWizard.
     */
    public ReqWizard() {
        super();
        setNeedsProgressMonitor(true);
    }

    /**
     * Adding the page to the wizard.
     */
    public void addPages() {
        page = new ReqWizardPage(selection);
        addPage(page);
    }

    @Override
    public boolean performFinish() {
        final String containerName = page.getContainerName();
        IWorkspaceRunnable createBase = new IWorkspaceRunnable() {
            @Override
            public void run(IProgressMonitor monitor) throws CoreException {
                try {
                    TreeDB db = TreesTracker.getTracker().getTree(newProject);
                    // Create nodes
                    TreeNode node = db.getRootNode();
                    node.createChild("Requirements", "Requirement");
                    node.createChild("Types", "NodeType");
                    node.createChild("Documents", "DocFolder");
                    node.createChild("Reports", "ReportFolder");
                } catch (Exception e) {
                    e.printStackTrace();
                } finally {
                    monitor.done();
                }
            }
        };
        try {
            IWorkspace workspace = ResourcesPlugin.getWorkspace();
            newProject = performFinishNewElement(containerName);
            workspace.run(createBase, null);
            updatePerspective();
        } catch (WorkbenchException e) {
            e.printStackTrace();
        } catch (CoreException e) {
            MessageDialog.openError(getShell(), "Error", e.getMessage());
            e.printStackTrace();
            return false;
        }
        return true;
    }

    public static IProject performFinishNewElement(String projectName) throws CoreException {
        try {
            IWorkspace workspace = ResourcesPlugin.getWorkspace();
            IWorkspaceRoot root = workspace.getRoot();
            IProject project = root.getProject(projectName);
            IProjectDescription desc = workspace
                    .newProjectDescription(projectName);
            desc.setNatureIds(new String[] { "com.unitesk.requality.natures.reqcatalogue" });
            project.create(desc, null);
            project.open(null);
            project.setDefaultCharset("UTF-8", null);
            return project;
        } catch (Exception e) {
            e.printStackTrace();
            return null;
        }
    }

    private void updatePerspective() throws WorkbenchException {
        IWorkbenchPage ap = PlatformUI.getWorkbench()
                .getActiveWorkbenchWindow().getActivePage();
        if (!ap.getPerspective().getId()
                .startsWith(REQUALITY_PERSPECTIVE_PREFIX)) {
            IPerspectiveRegistry reg = PlatformUI.getWorkbench()
                    .getPerspectiveRegistry();
            IPerspectiveDescriptor perspective = reg
                    .findPerspectiveWithId(ReqEditorPerspectiveFactory.ID);
            ap.setPerspective(perspective);
        }
    }

    /**
     * We will accept the selection in the workbench to see if we can initialize
     * from it.
     * 
     * @see IWorkbenchWizard#init(IWorkbench, IStructuredSelection)
     */
    public void init(IWorkbench workbench, IStructuredSelection selection) {
        this.selection = selection;
    }
}